/*
 * Copyright (c) 2017-2018 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *                                           Anthony Mallet on Mon Feb 27 2017
 */
#ifndef H_ARDUIO
#define H_ARDUIO

#include <stdarg.h>

#define F_CPU	16000000

/* pins */
#ifdef ARDUIO_UNO
# define ARIO_DPINS	14
# define ARIO_APINS	6
#elif defined(ARDUIO_NANO)
# define ARIO_DPINS	14
# define ARIO_APINS	8
#elif defined(ARDUIO_MEGA2560)
# define ARIO_DPINS	54
# define ARIO_APINS	16
#else
# error unknown board
#endif

enum __attribute__((__packed__)) ario_dir {
  ARIO_IN		= 0,
  ARIO_INPU		= 1,
  ARIO_OUT		= 2,
  ARIO_PWM_31kHz	= 3,
  ARIO_PWM_122Hz	= 4,
  ARIO_PWM_31Hz		= 5,
  ARIO_WS28XX		= 16
};

void		ario_pin_init(void);

void		ario_dpin_dir(uint8_t n, enum ario_dir dir);
void		ario_dpin_out(uint8_t n, uint16_t v);
uint16_t	ario_dpin_in(uint8_t n);
uint16_t	ario_apin_in(uint8_t n);

void		ario_ddir_state(void);
void		ario_dpin_state(void);
void		ario_apin_state(void);

void		ws28xx_set(volatile uint8_t *port, uint8_t bit, uint16_t v);


/* i/o */
#ifdef ARDUIO_UNO
# define ARDUIO_SEND_BUF	128
# define ARDUIO_RECV_BUF	64
#elif defined(ARDUIO_NANO)
# define ARDUIO_SEND_BUF	128
# define ARDUIO_RECV_BUF	64
#elif defined(ARDUIO_MEGA2560)
# define ARDUIO_SEND_BUF	256
# define ARDUIO_RECV_BUF	128
#endif

struct ario_msg {
  uint8_t len;
  uint8_t data[ARDUIO_RECV_BUF];
};

void			ario_uart_init(uint32_t baud);
void			ario_log(const char *fmt, ...);
void			ario_log_buffer(const uint8_t *buffer, uint8_t len);
struct ario_msg *	ario_recv(void);
void			ario_send(void);
void			ario_poll_uart(void);

#endif /* H_ARDUIO */
